#include <stdlib.h>
#include <stdio.h>
#include <windows.h>
#include <wincon.h>

#include <string.h>
#include <time.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <sys/timeb.h>
#include <assert.h>
#include <stdarg.h>


#include "vide.h"
#include "const.h"
#include "poz.h"
#include "jutil.h"
#include "chess.h"
#include "funlog.h"
#include "board2002.h"
#include "fovar.h"
#include "FunOcen.h"
#include "genpos.h"
#include "AABProc.h"
#include "WF1Proc.h"

#define MAX_POGLEB_SZACH 2    

#define KilerMovesMAX  3   

#ifdef _MSC_VER
#define inline __inline
#define CDECL __cdecl
#else
#define CDECL
#endif

#define     WHITE            1
#define     BLACK            0
#define     NO_COLOUR        2

#define     BOOL       	     int
#define     RESIGN_THRESHOLD  (1200)
#define		NOT_ALLOW_DRAW	(150)  // ZYZOL




typedef struct
   {
    PolruchListaWskT Front , Rear ;
    ushort Count ;
   } KilerMoveT ;

extern KilerMoveT   KilerMovesB ;
extern KilerMoveT   KilerMovesC ;

void DodajKM ( PolruchListaWskT M , KilerMoveT *KM ) ;

void DisposeKM (void) ;

void KilMvsInit (void) ;



extern ushort ABGlebokosc ;

int  AlfaBeta ( PozycjaTablicaWskT P ,
                ulong P_key ,
                int Alfa , int Beta ,
                ushort Glebokosc,
                int Pogleb ) ;
                


PolruchT AdamPos ;
int Adam ( PozycjaTablicaWskT P ) ;



ushort ABGlebokosc ;

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

typedef enum
   {
    NIEZNANY = 0 ,
    NIC ,
    PAT ,
    MAT
   } MatStatusT ;


/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

int Ocena ( PozycjaTablicaWskT P ) ;

bool GeneratorPosuniec ( PozycjaTablicaWskT P ,
                         PolruchListaWskT *ListaPosuniec ) ;

int RekAlfaBeta (PozycjaTablicaWskT P ,
                 ulong P_key ,
                 int Alfa , int Beta ,
                 ushort Glebokosc ,
                 PolruchListaWskT ListaPosuniecPom ,
                 MatStatusT *MatStatus,
                 int Pogleb ) ;

int ABKoniec ( PozycjaTablicaWskT P ,
               ulong P_key ,
               int Alfa , int Beta ,
               ushort Glebokosc ,
               int Best ,
               PolruchListaWskT *ListaPosuniec ,
               PolruchWskT ABPosPom );



/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

// Joanna 2002 does not have iterative deepening. Before each search it estimates
// a search depth, based on remaining time, game phase and no. of moves
// available for both sides. The formula has been tuned for the 2002 
// hardware, so it ought to be changed now.


int calculateDepth(void);

int calculateDepth()
{
	time_t now, t;
	int moveCount;
	int depth;
	PolruchListaWskT pos;

	
        /* the following code sums up the number of moves 
           generated for _both_ white and black  */

      
    pos = GeneratorPseudoPosuniecZwykly( &Pozycja );
	moveCount = CountMoves( pos );
	DisposePolruchLista( &pos );
	Pozycja.BialeNaPosunieciu = !Pozycja.BialeNaPosunieciu;

	pos = GeneratorPseudoPosuniecZwykly( &Pozycja );
	moveCount += CountMoves( pos );
	DisposePolruchLista( &pos );
	Pozycja.BialeNaPosunieciu = !Pozycja.BialeNaPosunieciu;


        /* now the formula: */	

        /* 1st case - time trouble */          

		if (time_control_9)   depth = 7;
        else
        if (time_control_8)   depth = 7;
        else
        if (time_control_7)   depth = 6;
        else
        if (time_control_6)   depth = 6;
        else
        if (time_control_5)   depth = 5;
        else
        if (time_control_4)   depth = 5;
        else
        if (time_control_3)   depth = 4;
        else
        if (time_control_2)   depth = 3;
        else
        if (time_control_1)   depth = 2;    

/* ZYZOL: 2nd case - Adam's idea, but little changed. 
In Adam's original code depth was increased by about 1 ply only in opening. Now if % 2 == 0
Now idea is used on other phases of games too. But a bit more rarely. */

		now = time( &t );
		if ( now % 3L == 0 )
		{
/* ZYZOL: 2nd case - opening */
			if ((FazaGry == Debiut) && (moveCount < 70))
			{
				depth += 1;
			}
/* ZYZOL: 2nd case - all phases */
			else if (moveCount < 60)
			{
				depth += 1;
			}
/* ZYZOL: 2nd case - all phases, small moveCount and more time */
			if ((moveCount < 45) && !time_control_1 && !time_control_2 && !time_control_3 && !time_control_4)
			{
				depth += 1;
			}
			if ((moveCount < 30) && !time_control_1 && !time_control_2 && !time_control_3)
			{
				depth += 1;
			}
		}

/* ZYZOL: 3nd case - simple endgame */
			if ((FazaGry == Koncowka) && (moveCount < 40) && !time_control_1 && !time_control_2 && !time_control_3)
			{
				depth += 1;
			}

/* 4rd case - we are not in the opening and we have ample amount of time */

/* ZYZOL: 4rd case - final cut */
	if ( count_moves < 7 )    /* In first seven moves engine save time to after use */
		depth = min(depth, 6);
	else
		depth = min(depth, 7);
	return depth;
}

int calculatePogleb()
{
    int pogleb = 2;
    if ( FazaGry == Matowanie || FazaGry == Debiut) // mating phase or opening ( = debiut)
		pogleb = 2; // in oryginal code pogleb = 0

	if ( FazaGry != Matowanie && FazaGry != Debiut) // If not mating phase or opening ( = debiut)
	{
		if (time_control_1 || time_control_2) pogleb = 1; 
		else
		if (time_control_3 || time_control_4 ) pogleb = 2; 
		else
   		if (time_control_5 || time_control_6 ) pogleb = 4;
		else
   		if (time_control_7 || time_control_8 || time_control_9) pogleb = 6;
	}
	return pogleb;
}



int Adam ( PozycjaTablicaWskT P )
{
    int pogleb;
	int   Value;
	PrepFunOcen ( P ) ;
	FazaGry = Faza ( P ) ;
	
	
	// Zamiast IDAlfaBeta
	Value = FunkcjaOceniajacaPoz ( P ) + WF1 ( P ) ;
    Value = AAlfaBeta ( P , 0UL/*P_key*/, Value , 2, 0 ) ; 
    
    my_max_depth = max_depth;
    max_depth = calculateDepth();
	pogleb = calculatePogleb();

/* ZYZOL move to calculatepogleb() function
	if (time_control_5 || time_control_6) pogleb = 4;
	else
    if (time_control_4 || time_control_3) pogleb = 3; 
*/   
    if (my_max_depth != max_depth) {
    printf("\nsearch depth adjusted to %d\n",max_depth);
    }
    
    Value = AAlfaBeta ( P , 0UL/*P_key*/, Value , max_depth, pogleb );
   
    DisposeFunOcen();
    
    Best_Score = Value;
	return Value;
}




int  AlfaBeta ( PozycjaTablicaWskT P ,
                ulong P_key , 
                int Alfa , int Beta ,
                ushort Glebokosc,
                int Pogleb )
{
 int Best , Value;
 PolruchListaWskT ListaPosuniec ;
 PolruchListaWskT ListaPosuniecPom ;
 PolruchWskT ABPosPom ;
 MatStatusT MatStatus = NIEZNANY ;
 int RekGlebokosc;
 
 

               
 if ( Glebokosc == 0 )
    return ( Ocena ( P ) ) ;
 else
    {
     if ( ! GeneratorPosuniec( P , &ListaPosuniec ) )
        {
         DisposePolruchLista ( &ListaPosuniec ) ;
         return ( BICIE_KROLA ) ;
        }

     if ( Szach( P ) && Pogleb > 0 )
         {
         RekGlebokosc = Glebokosc + 1;
         Pogleb--;
         }
     else
         {
         RekGlebokosc = Glebokosc;
         }
     

     Best = MININT; 
     ABPosPom = ListaPosuniecPom = ListaPosuniec ;
     while ( ListaPosuniecPom != NULL)
    
        {
         
        
         Value = RekAlfaBeta ( P, 0UL, MAX(Alfa,Best), Beta,
                               RekGlebokosc, ListaPosuniecPom, &MatStatus, Pogleb ) ;
                                    
        

         switch ( MatStatus )
            {
             case MAT :
                DisposePolruchLista ( &ListaPosuniec ) ;
                return ( - OCENAMATA + ( ABGlebokosc - Glebokosc ) ) ;

             case PAT :
                DisposePolruchLista ( &ListaPosuniec ) ;
                return ( 0 ) ;
            }
            
        

    
		if ( Value > Best )
            {
             ABPosPom = ListaPosuniecPom ;
             Best = Value ;
            
                       
                         
                         if ( Best >= Beta ) 
                         
                     
                         
                return ( ABKoniec ( P , P_key , Alfa , Beta , Glebokosc ,
                                    Best , &ListaPosuniec ,
                                    ABPosPom ) ) ;
                                    }
            
           
        
                    
        ListaPosuniecPom = ListaPosuniecPom->Nast ;
  
       
    }
    
    
    }


 return ( ABKoniec ( P , P_key , Alfa , Beta , Glebokosc ,
                     Best , &ListaPosuniec ,
                     ABPosPom ) ) ;
                     
                     
                     }
                       
                                   
                      



int Ocena ( PozycjaTablicaWskT P )
{
 int Wynik ;

 Wynik = WF1( P ) ;

 if ( Wynik == BICIE_KROLA )
    return ( Wynik ) ;
 else
    return ( FunkcjaOceniajacaPoz( P ) + Wynik ) ;
}

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

bool GeneratorPosuniec ( PozycjaTablicaWskT P ,
                         PolruchListaWskT *ListaPosuniec )
{
 *ListaPosuniec = GeneratorPseudoPosuniecZwykly ( P ) ;
 FunOcenGenPosVarsChange( P , *ListaPosuniec ) ;
 SortujAB ( P , ListaPosuniec ) ;
 if ( ( (*ListaPosuniec)->CoZbito == PoleBK ) ||
      ( (*ListaPosuniec)->CoZbito == PoleCK ) )
    return ( FALSE ) ;
 else
    return ( TRUE ) ;
}

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

int RekAlfaBeta (PozycjaTablicaWskT P ,
                 ulong P_key ,
                 int Alfa , int Beta ,
                 ushort Glebokosc ,
                 PolruchListaWskT ListaPosuniecPom ,
                 MatStatusT *MatStatus,
                 int Pogleb )
{
                               
 int Wynik ;
 PozycjaTablicaWskT PPrim ;

 PPrim = WykonajPolruch ( P , ListaPosuniecPom ) ;
 MoveFunOcen ( ListaPosuniecPom ) ;
 
 
             

 Wynik = - AlfaBeta( PPrim , 0UL ,
                     -Beta , -Alfa , Glebokosc - 1, Pogleb ) ;
                   
                   

 if ( Wynik == ( - BICIE_KROLA ) )
    switch ( *MatStatus )
      {
       case NIEZNANY :

            if ( Mat ( P ) )
               {
                *MatStatus = MAT ;
               }
            else if ( Pat ( P ) )
               {
                *MatStatus = PAT ;
               }
            else
               {
                *MatStatus = NIC ;
                Wynik = - OCENAMATA ; // Unikac podstawki krola !!
               }
            
            break;

       case NIC :
            
            Wynik = - OCENAMATA ; // Unikac podstawki krola !!
                   
            break ;
      }

 UnMoveFunOcen( ListaPosuniecPom ) ;
 free( PPrim ) ;


 return ( Wynik ) ;
}

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

int ABKoniec ( PozycjaTablicaWskT P ,
               ulong P_key ,
               int Alfa , int Beta ,
               ushort Glebokosc ,
               int Best ,
               PolruchListaWskT *ListaPosuniec ,
               PolruchWskT ABPosPom )
{
// CacheFlagT Flaga ;

 if ( Glebokosc == ABGlebokosc )
    AdamPos = *ABPosPom ;

 if ( ABGlebokosc - Glebokosc <= 2 )
    if ( P->BialeNaPosunieciu )
       DodajKM ( ABPosPom , &KilerMovesB ) ;
    else
       DodajKM ( ABPosPom , &KilerMovesC ) ;


 DisposePolruchLista ( ListaPosuniec ) ;

 return ( Best ) ;
}







char literka( PozycjaTablicaPoleT p );

char literka( PozycjaTablicaPoleT p )
{
	switch ( p )
		{
		case PoleBK    : return 'K' ;
      case PoleCK    : return 'k' ;
      case PoleBH    : return 'Q' ;
      case PoleCH    : return 'q' ;
      case PoleBW    : return 'R';
      case PoleCW    : return 'r' ;
      case PoleBG    : return 'B' ;
      case PoleCG    : return 'b' ;
      case PoleBS    : return 'N' ;
      case PoleCS    : return 'n' ;
      case PoleBP    : return 'P' ;
      case PoleCP    : return 'p' ;
	  default:
      case PustePole : return ' ' ;
	}
}



ShowJoannaMove()
{
	char ruch[20];  	          
	int i;          
	          if  (Best_Score > -RESIGN_THRESHOLD) {
                resigncounter = 0;
                }else{                                      
                if (resigncounter > 1) printf("\ thinking about resigning ..\n");
                resigncounter++;                
                }
                if (resigncounter == 3) {
                printf("joanna2002 resigns\n\n");
                printf("oh well, I knew I should've\n");
                printf("taken up checkers instead!!!\n\n");
				if (Engine_Colour == BLACK)
					printf("1-0 ");
				if (Engine_Colour == WHITE)
					printf("0-1 ");
				printf("{Joanna2002 resigns}\n");
                Engine_Colour = NO_COLOUR;
				return;
			};

	           if (Best_Score > 1000) {
                              printf(" thinking about winning!! ..\n");
                              }
			if ( AdamPos.PrzemianaPionkaW != PustePole )
					{
					i = sprintf(ruch, "%c%d%c%d%c", 'a' + AdamPos.Skad.LiniaPionowa, AdamPos.Skad.LiniaPozioma + 1,'a' + AdamPos.Dokad.LiniaPionowa, AdamPos.Dokad.LiniaPozioma + 1, literka( AdamPos.PrzemianaPionkaW ) );
					}
			else {
					i = sprintf(ruch, "%c%d%c%d", 'a' + AdamPos.Skad.LiniaPionowa, AdamPos.Skad.LiniaPozioma + 1,'a' + AdamPos.Dokad.LiniaPionowa, AdamPos.Dokad.LiniaPozioma + 1 );
			}

	          printf("%d %d %d %d %s\n",max_depth,Best_Score, 1, (int)avg_time_per_move, ruch);
/* ZYZOL where was data to time, nodes, pv?*/	          	
              printf("\n");              
              printf( "move %s\n", ruch);

             count_moves++;
}

void Przemiana( PolruchWskT pos, PozycjaTablicaPoleT kolor )
{
	
    
      
      
      switch ( buf[ 4 ] )
			{
			case 'q':
				pos->PrzemianaPionkaW = ( kolor == PoleBP ? PoleBH : PoleCH );
				break;
			case 'Q':
				pos->PrzemianaPionkaW = ( kolor == PoleBP ? PoleBH : PoleCH );
				break;	
			case 'r':
				pos->PrzemianaPionkaW = ( kolor == PoleBP ? PoleBW : PoleCW );
				break;
			case 'R':
				pos->PrzemianaPionkaW = ( kolor == PoleBP ? PoleBW : PoleCW );
				break;	
			case 'b':
				pos->PrzemianaPionkaW = ( kolor == PoleBP ? PoleBG : PoleCG );
				break;
			case 'B':
				pos->PrzemianaPionkaW = ( kolor == PoleBP ? PoleBG : PoleCG );
				break;	
			case 'n':
				pos->PrzemianaPionkaW = ( kolor == PoleBP ? PoleBS : PoleCS );
				break;
			case 'N':
				pos->PrzemianaPionkaW = ( kolor == PoleBP ? PoleBS : PoleCS );
				break;	
			}
   
}


void time_control() 
{
        
     if (max_time <= 1200) {  
            time_control_1  = TRUE;
            time_control_2 = time_control_3 = time_control_4 = time_control_5 = 
            time_control_6 = time_control_7 = time_control_8 = time_control_9 = false;
            printf("[time control 1]\n");          
            }else
            if (max_time <= 4000) {  
            time_control_2  = TRUE;
            time_control_1 = time_control_3 = time_control_4 = time_control_5 = 
            time_control_6 = time_control_7 = time_control_8 = time_control_9 = false;
            printf("[time control 2]\n");
            }else
            if (max_time <=  15000) {  
            time_control_3  = TRUE;
            time_control_1 = time_control_2 = time_control_4 = time_control_5 = 
            time_control_6 = time_control_7 = time_control_8 = time_control_9 = false;
            printf("[time control 3]\n");
            }else
            if (max_time <= 30000) {  
            time_control_4  = TRUE;
            time_control_1 = time_control_2 = time_control_3 = time_control_5 = 
            time_control_6 = time_control_7 = time_control_8 = time_control_9 = false;
            printf("[time control 4]\n");
            }else
            if (max_time <= 48000) {  
            time_control_5  = TRUE;
            time_control_1 = time_control_2 = time_control_3 = time_control_4 = 
            time_control_6 = time_control_7 = time_control_8 = time_control_9 = false;
            printf("[time control 5]\n");
            }else
            if (max_time <= 90000) {  
			time_control_6  = TRUE;
            time_control_1 = time_control_2 = time_control_3 = time_control_4 = 
            time_control_5 = time_control_7 = time_control_8 = time_control_9 = false;
            printf("[time control 6]\n");
			}else
            if (max_time <= 160000) {  
            time_control_7  = TRUE;
            time_control_1 = time_control_2 = time_control_3 = time_control_4 = 
            time_control_5 = time_control_6 = time_control_8 = time_control_9 = false;
            printf("[time control 7]\n");
            }else
            if (max_time <= 280000) {  
            time_control_8  = TRUE;
            time_control_1 = time_control_2 = time_control_3 = time_control_4 = 
            time_control_5 = time_control_6 = time_control_7 = time_control_9 = false;
            printf("[time control 8]\n");
            }else{
            time_control_9  = TRUE;
            time_control_1 = time_control_2 = time_control_3 = time_control_4 = 
            time_control_5 = time_control_6 = time_control_7 = time_control_8 = false;
            printf("[time control 9]\n");
           
            }
    
        }
	
void UstawianiezFen();

void ReadOppenentMove()
{
	PolruchT pos;
		
            
	printf("");
    signal(SIGINT, catch_sigint);
    printf("");	  	 


    for (;;)
    {
	  
      if (!fgets(line, 256, stdin)) return;
	  if (line[0] == '\n') continue;
		
      sscanf(line, "%s", buf); 

    if (!strcmp(buf, "quit") || !strcmp(buf, "exit")) {
            printf("\nJoanna2002 says 'Do widzenia!' {goodbye}\n");	
	        exit(0);
			};
						
                              

          	if (!strcmp(buf, "go")) {
			  Engine_Colour = Pozycja.BialeNaPosunieciu ;
/* ZYZOL ....... Engine_Colour = side_to_move; */

            Board2002();
            }
                                         
     
         if (!strcmp(buf, "force")) {
         
            Engine_Colour = NO_COLOUR;
          	continue;
			};
     
     if (!strcmp(buf, "setboard")) {
			FunOcenInit();
            InitP();     
            Engine_Colour = BLACK;
			UstawianiezFen();
			continue;
			};
    
     if (!strcmp(buf, "new")) {
            FunOcenInit();
            InitP();     
            Engine_Colour = BLACK;   
          	continue;
			};
			
			
		if (!strcmp(buf, "white")) {
            side_to_move = WHITE;
            Engine_Colour = BLACK;
            continue;
			};
				
		if (!strcmp(buf, "black")) {
            side_to_move = BLACK;
            Engine_Colour = WHITE;
            continue;
			};		
			
			
					
			
	if (!strcmp(buf, "protover")) {
              printf("feature colors=1 myname=\"Joanna2002 JA %s by Adam Kujawski\"",VERSION);
              printf(" time=1 pause=0 variants=\"normal\" sigint=0 sigterm=0");
              printf(" name=1 reuse=0 playother=1 variant=1 setboard=1 draw=1 done=1\n");
              continue;
              }
              
              
              
       if (!strcmp(buf,"variant")) {
	    	char variant[100];
	    	scanf("%s",variant);
	    	if (!strcmp(variant,"normal")) {
			printf("Error: unsupported variant.");
			continue;
	    	}
          }
               
                         
          if (!strcmp(buf, "sd")) {
			sscanf(line, "sd %d", &max_depth);
			max_time = 1000000;
			continue;
		}

                   
          
            if (!strcmp(buf, "st")) {
	        sscanf(line, "st %d", &max_time);
            max_time *= 1000;
            max_depth = 10000;
			continue;                    
            } 
 
         
        if (!strcmp(buf, "time")) {       
            sscanf(line, "time %d", &max_time);
            time_control();
			continue;
           }
	
        if (!strcmp(buf, "otim")) {       
            sscanf(line, "otim %d", &otim);
			continue;
           }


          if (!strcmp(buf, "result")) {
           
           char result[256];
           sscanf(line,"result %s",result);
           Engine_Colour = NO_COLOUR;
   	       continue;
           }		


     
     if (!strcmp(buf, "xboard")) {
         
   		    xboard = TRUE;
            printf("feature done=0\n");
		    continue;          
			};



if (!strcmp(buf, "post")) {
  continue;
		};
if (!strcmp(buf, "nopost")) {
          continue;
		};
if (!strcmp(buf, ".")) {
         continue;
		};
if (!strcmp(buf, "?")) {
         continue;
		};
if (!strcmp(buf, "random")) {
         continue;
		};	
if (!strcmp(buf, "hard")) {
         continue;
		};		      	
if (!strcmp(buf, "easy")) {
         continue;
		};		      
if (!strcmp(buf, "computer")) {
         continue;
		};		  
if (!strcmp(buf, "accepted")) {
         continue;
		};		          
if (!strcmp(buf, "rejected")) {
         continue;
		};		                 
   
   
       	if ( buf[ 0 ] < 'a' || buf [ 0 ] > 'h' )
			{ continue; }
		if ( buf[ 1 ] < '1' || buf [ 1 ] > '8' )
			{ continue; }
		if ( buf[ 2 ] < 'a' || buf [ 2 ] > 'h' )
			{ continue; }
		if ( buf[ 3 ] < '1' || buf [ 3 ] > '8' )
			{ continue; }

		pos.Skad.LiniaPionowa = buf[ 0 ] - 'a';
		pos.Skad.LiniaPozioma = buf[ 1 ] - '1';
		pos.Dokad.LiniaPionowa = buf[ 2 ] - 'a'; 
		pos.Dokad.LiniaPozioma = buf[ 3 ] - '1';
		pos.Co = Pozycja.Pozycja[ pos.Skad.LiniaPionowa][ pos.Skad.LiniaPozioma ]; 
		pos.CoZbito = Pozycja.Pozycja[ pos.Dokad.LiniaPionowa][ pos.Dokad.LiniaPozioma ];
		pos.PrzemianaPionkaW = PustePole;

		if ( pos.Co == PoleBP && pos.Dokad.LiniaPozioma == 7 )
			{
			Przemiana( &pos, PoleBP );
			}
		if ( pos.Co == PoleCP && pos.Dokad.LiniaPozioma == 0 )
			{
			Przemiana( &pos, PoleCP );
			}
		if ( ( pos.Co == PoleBP || pos.Co == PoleCP ) 
			&& pos.Skad.LiniaPionowa != pos.Dokad.LiniaPionowa 
			&& Pozycja.Pozycja[ pos.Dokad.LiniaPionowa ][ pos.Dokad.LiniaPozioma ] == PustePole )
			{
			pos.EP = TRUE;
			pos.CoZbito = pos.Co == PoleBP ? PoleCP : PoleBP;
			}
		else
			pos.EP = FALSE;



		if ( !Mozliwy( &Pozycja, &pos ) )
			{ continue; }
		break;

		}
		
		
  MakeMove( &pos );
  
            
}



void KoniecGry()
{
if ( Mat ( &Pozycja ) )
//	if ( Pozycja.BialeNaPosunieciu &&  Engine_Colour == WHITE  || !Pozycja.BialeNaPosunieciu && Engine_Colour == BLACK)
	{  
		if (Pozycja.BialeNaPosunieciu &&  Engine_Colour == WHITE)
			printf("0-1 {Joanna2002 mated}\ngood things come to\nhe who waits !!! \n");
		if (!Pozycja.BialeNaPosunieciu && Engine_Colour == BLACK)
			printf("1-0 {Joanna2002 mated}\ngood things come to\nhe who waits !!! \n");
		if (Pozycja.BialeNaPosunieciu &&  Engine_Colour == BLACK)
			printf("0-1 {Joanna2002 mates}\ngood things come to\nhe who waits !!! \n");
		if (!Pozycja.BialeNaPosunieciu && Engine_Colour == WHITE)
			printf("1-0 {Joanna2002 mates}\ngood things come to\nhe who waits !!! \n");
	 Engine_Colour = NO_COLOUR;
  
	 return;
	}
 else if ( Pat ( &Pozycja ) )
    {

     printf("1/2-1/2 {stalemate}\ngame is a draw\n");
     Engine_Colour = NO_COLOUR;
	 return;
    }
 else if ( RuleFiftyMovesCounter >= 100 && ( Best_Score < NOT_ALLOW_DRAW || (max_time * 3) < (otim * 2)) )
    {
	
    printf("1/2-1/2 {draw by fifty moves rule}\ngame is a draw\n");
    Engine_Colour = NO_COLOUR;     
    return;
    }
 else if ( RuleFiftyMovesCounter >= 8 )
	{
		 if ( TrzykrotnePowtorzeniePozycji () && ( (Best_Score < NOT_ALLOW_DRAW) || (max_time * 3) < (otim * 2)) )
		{
		
		  printf("1/2-1/2 {draw by repetition}\ngame is a draw\n");
		  Engine_Colour = NO_COLOUR;    
  		 return;
		}
	}
return;
}



/* ZYZOL based on funktion ustawianie() Adama Kujawskiego */

void UstawianiezFen()
{
	int i, j, k;
	TCHAR buf[ 100 ];
	char line1[256];
	char pb[100], cr[2], mr[5], mbp[3];
	int lprpb, lpr; 
/* First step eliminate quote, doublequote and "`" sign */
	i=0;
	j=0;
	do
	{
		if(line[i]!='\`' && line[i]!='\'' && line[i]!='\"')
		{
			line1[j] = line[i];
			j++;
		}
		i++;
	}
	while(line[i] > 0);
	line1[j]=0;

	sscanf(line1, "setboard %s %s %s %s %d %d", &pb, cr, &mr, &mbp, &lprpb, &lpr );

	
/* ZYZOL: piece placement */
/* ZYZOL: empty board */
	for ( j = 0 ; j <= 7 ; j++ )
		for ( i = 0 ; i <= 7 ; i++ )
			Pozycja.Pozycja[ i ][ j ] = PustePole;

/* ZYZOL: Structuring of pawn according to pb string */
	i = 0 ; /* pionowe , vertical, a-h */
	j = 7 ; /* poziome, horizontal, 1-8 */
	for ( k = 0 ; k <= strlen(pb) ; k++ )
	{
		if((pb[k] >= '0') && (pb[k] <= '8'))
		{
			i = i + pb[k] - '0';
			continue;
		}
		if(pb[k] == '/')
		{
			j--;
			i = 0;
			continue;
		}
		switch ( pb[k] )
			{
			case 'P': Pozycja.Pozycja[ i ][ j ] = PoleBP; 
				break;
			case 'p': Pozycja.Pozycja[ i ][ j ] = PoleCP; 
				break;
			case 'K': Pozycja.Pozycja[ i ][ j ] = PoleBK; 
				break;
			case 'k': Pozycja.Pozycja[ i ][ j ] = PoleCK; 
				break;
			case 'Q': Pozycja.Pozycja[ i ][ j ] = PoleBH; 
				break;
			case 'q': Pozycja.Pozycja[ i ][ j ] = PoleCH; 
				break;
			case 'R': Pozycja.Pozycja[ i ][ j ] = PoleBW; 
				break;
			case 'r': Pozycja.Pozycja[ i ][ j ] = PoleCW; 
				break;
			case 'B': Pozycja.Pozycja[ i ][ j ] = PoleBG; 
				break;
			case 'b': Pozycja.Pozycja[ i ][ j ] = PoleCG; 
				break;
			case 'N': Pozycja.Pozycja[ i ][ j ] = PoleBS; 
				break;
			case 'n': Pozycja.Pozycja[ i ][ j ] = PoleCS; 
				break;
			default:
				break;
			}
		i++;
	}


/* ZYZOL: side to move */
	if(cr[0] == 'w' || cr[0] == 'W')
			Pozycja.BialeNaPosunieciu = TRUE;
	else
			Pozycja.BialeNaPosunieciu = FALSE;


/* ZYZOL: castling */
	Pozycja.KrotkaRoszadaBialychMozliwa = FALSE;
	Pozycja.KrotkaRoszadaCzarnychMozliwa = FALSE;
	Pozycja.DlugaRoszadaBialychMozliwa = FALSE;
	Pozycja.DlugaRoszadaCzarnychMozliwa = FALSE;

	for ( i = 0 ; i <= strlen(mr) ; i++ )
	{
		if(mr[i] == '-')
			break;
		if(mr[i] == 'K')
			Pozycja.KrotkaRoszadaBialychMozliwa = TRUE;
		if(mr[i] == 'k')
			Pozycja.KrotkaRoszadaCzarnychMozliwa = TRUE;
		if(mr[i] == 'Q')
			Pozycja.DlugaRoszadaBialychMozliwa = TRUE;
		if(mr[i] == 'q')
			Pozycja.DlugaRoszadaCzarnychMozliwa = TRUE;


	}
/* ZYZOL En passant */
	if ( mbp[0] == '-')
	{
     Pozycja.EPMozliwe = FALSE ;
     Pozycja.EPLinia = (LiniaPionowaT)8 ;
	}
	else
	{
     Pozycja.EPMozliwe = TRUE ;
     Pozycja.EPLinia = mbp[0] - 'a' ;
	}

/* ZYZOL Halfmove clock */
	RuleFiftyMovesCounter = lprpb;
/* ZYZOL Fullmove number */
	count_moves = lpr - 1 ; /* ZYZOL: engine counts moves begining with 0 */

}


Board2002()
{
	time_t start, finish, now;

	for (;;)
		{
	start = time(&now);
			if ( Pozycja.BialeNaPosunieciu &&  Engine_Colour == WHITE  || !Pozycja.BialeNaPosunieciu && Engine_Colour == BLACK)
			{   
               
                      
            printf("\nJoanna2002 is thinking....\n");                             
       
            KoniecGry();
                                                
		
            	          
            CallJoanna();
			
            
            ShowJoannaMove();
            
	finish = time(&now);
	max_time -= 100 * difftime(finish, start);
          
            KoniecGry();  
			
		   }
        else
			{
			ReadOppenentMove();
			
		//	KoniecGry();  
		
			}
					

side_to_move = WHITE ? BLACK : WHITE;

		}
		
		
return 0;
}

///////////////////////////////////////////////////////////

